@@ -1,4 +1,5 @@
 # import {{{1
+import os
 import json
 import pyexcel
 
@@ -12,20 +13,22 @@ from django.shortcuts import render, redirect
 from django.views.generic import DetailView, CreateView, UpdateView, DeleteView, ListView
 from django.views.generic.base import View, TemplateView
 from django.views.generic.edit import FormView
+
 from django.contrib.auth.mixins import PermissionRequiredMixin
 from django.contrib.auth import views as auth_views
+
 from django.utils.translation import gettext_lazy as _
 
+from rest_framework.views import APIView
+from rest_framework.response import Response
+from rest_framework import status
+from rest_framework.permissions import IsAdminUser
+
 from .forms import get_generic_form, ImportXlsForm
 from .core import get_color_from_class, get_layout_data, get_model_fields
-from .models import GenericModel
+from .models import GenericModel, GenericUser, GlobalHistory
+from .serializers import GenericUserSerializer
 
-# LOGIN {{{1
-class GenericLoginView(auth_views.LoginView):
-    extra_context = {
-        'settings': settings,
-        'test': 'test'
-    }
 
 # DASHBOARD {{{1
 class GenericDashboardView(PermissionRequiredMixin, TemplateView):
@@ -64,6 +67,7 @@ class GenericDashboardView(PermissionRequiredMixin, TemplateView):
 # class GenericOmnibarView(PermissionRequiredMixin, View):
 class GenericOmnibarView(View):
     models = []
+    data = []
     auth = 'superuser'
 
     def __init__(self, **kwargs):
@@ -82,6 +86,16 @@ class GenericOmnibarView(View):
                     'param0': o.get_omnibar_ref(),
                     'param1': o.get_absolute_url()
                 })
+
+        for f in self.data:
+            with open(f) as data_file:
+                data = json.load(data_file)
+                for item in data:
+                    item_d = {}
+                    for k,v in item.items():
+                        item_d[k] = v
+                    d.append(item_d)
+
         return JsonResponse(d, safe=False)
         
 # LIST {{{1
@@ -90,12 +104,14 @@ class GenericListView(PermissionRequiredMixin, ListView):
     template_list_name = 'genviews/table.html'
     list_cls = 'generic-list'
     fields = '__all__' 
+    ordering = ['pk']
 
     def __init__(self, **kwargs):
-        assert hasattr(settings, 'FIELDMAP'), f"Pas de template FIELDMAP défini dans settings.py, il faut définir comment chaque field de la liste va s'afficher dans un template"
+        assert hasattr(settings, 'LIST_FIELDMAP'), f"Pas de template LIST_FIELDMAP défini dans settings.py, il faut définir comment chaque field de la liste va s'afficher dans un template"
         self.current_app = self.model._meta.app_label
         self.permission_required = f'{self.current_app}.list_{self.model.__name__.lower()}'
         self.template_name = self.model.get_view_data('list', 'tpl') or self.template_name
+        self.ordering = self.model.get_view_data('list', 'ordering') or self.ordering
         super().__init__(**kwargs)
 
     def get_context_data(self, **kwargs):
@@ -104,20 +120,20 @@ class GenericListView(PermissionRequiredMixin, ListView):
         context['list_title'] = self.model.get_view_data('list', 'title')
         context['list_menu'] = self.model.get_list_menu()
         context['list_cls'] = self.list_cls
-        context['fields'] = self.get_list_fields();
+        context['fields'] = self._get_fields();
         context['html'] = self.model.get_view_data('list', 'html') or self.template_list_name
 
         context['settings'] = settings
         context['layout_data'] = get_layout_data()
         return context
 
-    def get_list_fields(self):
-        view_struct_fields = self.model.get_view_data('list')
-        return get_model_fields(self.model, view_struct_fields)
+    def _get_fields(self):
+        fields = self.model.get_view_data('list', 'fields')
+        return get_model_fields(self.model, fields)
 
 # CREATE {{{1
 class GenericCreateView(PermissionRequiredMixin, CreateView):
-    template_name = 'genviews/form.html'
+    template_name = 'genviews/form_view.html'
     color_classes = []
 
     def __init__(self, **kwargs):
@@ -136,6 +152,25 @@ class GenericCreateView(PermissionRequiredMixin, CreateView):
         view_struct_fields = self.model.get_view_data('create')
         return get_generic_form(self.model, view_struct_fields) 
 
+    def form_valid(self, form):
+        if hasattr(self.model, 'history'):
+            print('historized')
+            new_object = form.save()
+            self.object = new_object
+            # old_tags = set(Tag.objects.filter(equipments=equipment))
+            # new_tags = set(form.cleaned_data['tags'])
+            new_object.history_create(
+                self.request.user, 
+            )
+                # True, 
+                # form.cleaned_data['change_reason'])
+                # (old_tags, new_tags)
+            # form.save_m2m()
+            return redirect(self.get_success_url())
+
+        print('not historized')
+        return super().form_valid(form)
+
     def get_context_data(self, **kwargs):
         if 'param' in self.kwargs and 'pk' in self.kwargs:
             self.initial = {
@@ -164,7 +199,7 @@ class GenericCreateView(PermissionRequiredMixin, CreateView):
 
 # UPDATE {{{1
 class GenericUpdateView(PermissionRequiredMixin, UpdateView):
-    template_name = 'genviews/form.html'
+    template_name = 'genviews/form_view.html'
     color_classes = []
 
     def __init__(self, **kwargs):
@@ -184,6 +219,24 @@ class GenericUpdateView(PermissionRequiredMixin, UpdateView):
         view_struct_fields = self.model.get_view_data('update')
         return get_generic_form(self.model, view_struct_fields) 
 
+    def form_valid(self, form):
+        if hasattr(self.model, 'history'):
+            print('historized')
+            self.object = form.save()
+            # old_tags = set(Tag.objects.filter(equipments=equipment))
+            # new_tags = set(form.cleaned_data['tags'])
+            self.object.history_update(
+                self.request.user, 
+            )
+                # True, 
+                # form.cleaned_data['change_reason'])
+                # (old_tags, new_tags)
+            # form.save_m2m()
+            return redirect(self.get_success_url())
+
+        print('not historized')
+        return super().form_valid(form)
+
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
 
@@ -212,6 +265,7 @@ class GenericDetailView(PermissionRequiredMixin, DetailView):
     template_name = 'genviews/detail.html'
 
     def __init__(self, **kwargs):
+        assert hasattr(settings, 'DETAIL_FIELDMAP'), f"Pas de template DETAIL_FIELDMAP défini dans settings.py, il faut définir comment chaque field de la vue de détail va s'afficher dans un template"
         self.current_app = self.model._meta.app_label
         self.permission_required = f'{self.current_app}.detail_{self.model.__name__.lower()}'
         self.template_name = self.model.get_view_data('detail', 'tpl') or self.template_name
@@ -220,21 +274,22 @@ class GenericDetailView(PermissionRequiredMixin, DetailView):
     def get_context_data(self, **kwargs):
         context = super().get_context_data(**kwargs)
 
-        context['html'] = self.model.get_view_data('detail', 'html') or None
-        context['MEDIA_URL'] = settings.MEDIA_URL
         context['detail_menu'] = self.object.get_view_menu('detail')
+        context['fields'] = self._get_fields()
 
-        # TODO utile ??
-        #if hasattr(self, 'actions_extra'):
-        #    context['actions_extra'] = self.actions_extra
-
-        #if hasattr(self, 'template_extra'):
-        #    context['template_extra'] = self.template_extra
+        context['html'] = self.model.get_view_data('detail', 'html') or None
 
         context['settings'] = settings
+        context['MEDIA_URL'] = settings.MEDIA_URL
         context['layout_data'] = get_layout_data()
         return context
 
+    def _get_fields(self):
+        fields = self.model.get_view_data('detail', 'fields')
+        if fields == '__all__':
+            return None
+        return fields
+
 # HISTORY {{{1
 class GenericHistoryView(PermissionRequiredMixin, DetailView):
     template_name = 'genviews/history.html'
@@ -428,6 +483,82 @@ class GenericImportxlssaveView(PermissionRequiredMixin, View):
 #      # return HttpResponse(xls_file, content_type="application/vnd.ms-excel")
 #   pass
 
+# LOGIN {{{1
+class GenericLoginView(auth_views.LoginView):
+    extra_context = {
+        'settings': settings,
+        'test': 'test'
+    }
+
+# LOGOUT {{{1
+class GenericLogoutView(auth_views.LogoutView):
+    extra_context = {
+        'settings': settings,
+        'test': 'test'
+    }
+# PROFILE {{{1
+class GenericProfileView(UpdateView):
+    template_name = 'auth/user_detail.html'
+    model = GenericUser
+    fields = [
+        'first_name', 'last_name', 'email', 'style'
+    ]
+    extra_context = {
+        'settings': settings,
+        'test': 'test'
+    }
+
+    def __init__(self, **kwargs):
+        self.current_app = self.model._meta.app_label
+        self.permission_required = f'{self.current_app}.detail_{self.model.__name__.lower()}'
+        super().__init__(**kwargs)
+
+    def get_context_data(self, **kwargs):
+        context = super().get_context_data(**kwargs)
+
+        context['settings'] = settings
+        context['layout_data'] = get_layout_data()
+
+        context['history'] = GlobalHistory.objects.filter(
+            user=self.object
+        )
+        user_form = get_generic_form(self.model, [
+            'username', 'first_name', 'last_name', 'email', 'style'
+        ])
+        # context['user_form'] = user_form(instance=self.request.user)
+        return context
+
+# test flutter {{{1
+class UserRecordView(APIView):
+    """
+    API View to create or get a list of all the registered
+    users. GET request returns the registered users whereas
+    a POST request allows to create a new user.
+    """
+    permission_classes = [IsAdminUser]
+
+    def get(self, format=None):
+        users = GenericUser.objects.all()
+        serializer = GenericUserSerializer(users, many=True)
+        return Response(serializer.data)
+
+    def post(self, request):
+        serializer = GenericUserSerializer(data=request.data)
+        if serializer.is_valid(raise_exception=ValueError):
+            serializer.create(validated_data=request.data)
+            return Response(
+                serializer.data,
+                status=status.HTTP_201_CREATED
+            )
+
+        return Response(
+            {
+                "error": True,
+                "error_msg": serializer.error_messages,
+            },
+            status=status.HTTP_400_BAD_REQUEST
+        )
+
 # create_model_paths {{{1
 def create_model_paths(model):
     mod_name = model.__name__
